﻿using System;
using System.Collections.ObjectModel;
using System.Linq;
using System.Windows.Forms;
using ThinkGeo.MapSuite;
using ThinkGeo.MapSuite.Drawing;
using ThinkGeo.MapSuite.Layers;
using ThinkGeo.MapSuite.Shapes;
using ThinkGeo.MapSuite.Styles;
using ThinkGeo.MapSuite.WinForms;

namespace CSHowDoISamples
{
    public class CustomLegendItem : LegendItem
    {
        protected override void DrawCore(GeoCanvas adornmentGeoCanvas, Collection<SimpleCandidate> labelsInAllLayers, LegendDrawingParameters legendDrawingParameters)
        {
            var imageStyle = ImageStyle;
            ImageStyle = null;
            base.DrawCore(adornmentGeoCanvas, labelsInAllLayers, legendDrawingParameters);
            ImageStyle = imageStyle;
            if (ImageStyle != null)
            {
                float newImageWidth = this.Ratio * this.ImageWidth;
                // use ImageHeight instead of ImageWidth
                float newImageHeight = this.Ratio * this.ImageHeight;

                float totalImageTopPaddingPixel = legendDrawingParameters.YOffset + TopPadding + ImageTopPadding;
                float totalImageLeftPaddingPixel = legendDrawingParameters.XOffset + LeftPadding * this.Ratio + ImageLeftPadding * this.Ratio;

                DrawingRectangleF imageExtent = new DrawingRectangleF(newImageWidth / 2 + totalImageLeftPaddingPixel, newImageHeight / 2 + totalImageTopPaddingPixel * this.Ratio, newImageWidth, newImageHeight);
                if (ImageMask != null)
                {
                    ImageMask.DrawSample(adornmentGeoCanvas, imageExtent);
                }
                if (imageStyle is PointStyle)
                {
                    (imageStyle as PointStyle).ImageScale = this.Ratio;
                }
                ImageStyle.DrawSample(adornmentGeoCanvas, imageExtent);
            }
        }
    }

    public partial class LegendForm : Form
    {
        public LegendForm()
        {
            InitializeComponent();
        }

        private void LegendForm_Load(object sender, EventArgs e)
        {
            //mapView.MapTools.PanZoomBar.IsEnabled = false;
            this.WindowState = FormWindowState.Maximized;
            mapView.MapUnit = GeographyUnit.Meter;

            var housingUnitsLayer = new ShapeFileFeatureLayer(@".\Data\Frisco 2010 Census Housing Units.shp");
            var legend = new LegendAdornmentLayer
            {
                Height = 800,
                Title = new CustomLegendItem()
                {
                    TextStyle = new TextStyle("Housing Units", new GeoFont("Verdana", 10, DrawingFontStyles.Bold), GeoBrushes.Black)
                },
                Location = AdornmentLocation.LowerRight
            };

            mapView.AdornmentOverlay.Layers.Add(legend);

            housingUnitsLayer.FeatureSource.Projection = new Proj4Projection(2276, 3857);
            //ProjectionConverter = new ProjectionConverter(2276, 3857);

            AddClassBreakStyle(housingUnitsLayer, legend);

            var layerOverlay = new LayerOverlay();
            layerOverlay.Layers.Add(housingUnitsLayer);

            mapView.Overlays.Add(layerOverlay);

            housingUnitsLayer.Open();
            mapView.CurrentExtent = housingUnitsLayer.GetBoundingBox();
            housingUnitsLayer.Close();

            mapView.Refresh();

        }

        private void AddClassBreakStyle(ShapeFileFeatureLayer layer, LegendAdornmentLayer legend)
        {
            var housingUnitsStyle = new ClassBreakStyle("H_UNITS");

            var classBreakIntervals = new double[] { 0, 100, 200, 300, 400, 500, 600, 700, 800, 900, 1000, 1100, 1200, 1300, 1400, 1500,
                1600, 1700, 1800, 1900, 2000, 2100, 2200, 2300, 2400, 2500, 2600, 2700, 2800, 2900, 3000,
                3100, 3200, 3300, 3400, 3500, 3600, 3700, 3800, 3900, 4000, 4100, 4200, 4300, 4400, 4500, 4600, 4700, 4800, 4900, 5000 };

            var colors = GeoColor.GetColorsInHueFamily(GeoColors.Red, classBreakIntervals.Count()).Reverse().ToList();

            for (int i = 0; i < classBreakIntervals.Length; i++)
            {
                var color = new GeoColor(255, colors[i]);
                var classBreak = new ClassBreak(classBreakIntervals[i], new AreaStyle(new GeoPen(color, 0), new GeoSolidBrush(color)));

                housingUnitsStyle.ClassBreaks.Add(classBreak);

                var legendItem = new CustomLegendItem()
                {
                    ImageStyle = classBreak.DefaultAreaStyle,
                    Width = 60,
                    ImageWidth = 21,
                    ImageTopPadding = 0,
                    ImageBottomPadding = 0,
                    // ImageHeight should be Height + 1 to completely fill the vertical space
                    ImageHeight = 16,
                    Height = 15,
                    // use the proper font size to ensure it does not exceed the LegendItem's height
                    TextStyle = new TextStyle($@">{classBreak.Value} units", new GeoFont("Verdana", 8), GeoBrushes.Black)
                };
                legend.LegendItems.Add(legendItem);
            }

            layer.ZoomLevelSet.ZoomLevel01.CustomStyles.Add(housingUnitsStyle);
            layer.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;
        }
    }
}
